// Find the N most frequently occurring words in the standard input 
// and print them and their number of occurrences.
#include <string>     // Include standard string class
#include <iostream>    // Include standard stream I/O classes
#include <map>         // Include STL map and multimap containers
#include <algorithm>   // Include STL generic algorithms
#include <numeric>     // Include STL generalized numeric algorithms
#include "stringhelp.h"  // Include string help functions
#include "pairhelp.h"    // Include pair help functions
using namespace std;

int main(int argc, char* argv[])
{
  int N = atoi(argv[1]);  // Number of the most frequent words to find

  typedef map<string, int> 
    frequency_map;  // Type of table that holds words and frequencies
  typedef istream_iterator<string>
    string_input;   // Type of iterator for traversing an input stream 

  frequency_map fm;
  for (string_input j(cin); j != string_input(); ++j) 
      ++fm[normalize(*j)];  // Increment frequency of normalized string
  /* 
  for (frequency_map::iterator p = fm.begin(); p != fm.end(); ++p)
    cout << (*p).first << "\t" << (*p).second << endl;
  cout << endl;
  */
  int M = accumulate(fm.begin(), fm.end(), 0, plus2<const string, int>());

  cout << "words: " << M << ", distinct words: " << fm.size() << "\n" 
       << N << " most frequent words:" << endl;

  typedef multimap<int, string, greater<int> > 
    reverse_frequency_map; // Type of table that holds frequencies and 
                           // words in nonincreasing order of frequency
  reverse_frequency_map rf;
  for (frequency_map::iterator i = fm.begin(); i != fm.end(); ++i)
    rf.insert(reverse_pair(*i));

  int count = 0;
  reverse_frequency_map::iterator k;
  for (k = rf.begin(); k != rf.end() && count++ < N; ++k)
    cout << (*k).first << "\t " << (*k).second << "\n";
  return 0;
}
